﻿/******************* 
 * Memorytask Test *
 *******************/

import { core, data, sound, util, visual } from './lib/psychojs-2022.1.3.js';
const { PsychoJS } = core;
const { TrialHandler, MultiStairHandler } = data;
const { Scheduler } = util;
//some handy aliases as in the psychopy scripts;
const { abs, sin, cos, PI: pi, sqrt } = Math;
const { round } = util;


// store info about the experiment session:
let expName = 'MemoryTask';  // from the Builder filename that created this script
let expInfo = {'participant': '', 'session': '001'};

// Start code blocks for 'Before Experiment'
// init psychoJS:
const psychoJS = new PsychoJS({
  debug: true
});

// open window:
psychoJS.openWindow({
  fullscr: false,
  color: new util.Color([0,0,0]),
  units: 'height',
  waitBlanking: true
});
// schedule the experiment:
psychoJS.schedule(psychoJS.gui.DlgFromDict({
  dictionary: expInfo,
  title: expName
}));

const flowScheduler = new Scheduler(psychoJS);
const dialogCancelScheduler = new Scheduler(psychoJS);
psychoJS.scheduleCondition(function() { return (psychoJS.gui.dialogComponent.button === 'OK'); }, flowScheduler, dialogCancelScheduler);

// flowScheduler gets run if the participants presses OK
flowScheduler.add(updateInfo); // add timeStamp
flowScheduler.add(experimentInit);
flowScheduler.add(ExperimentInitRoutineBegin());
flowScheduler.add(ExperimentInitRoutineEachFrame());
flowScheduler.add(ExperimentInitRoutineEnd());
flowScheduler.add(WelcomeScreenRoutineBegin());
flowScheduler.add(WelcomeScreenRoutineEachFrame());
flowScheduler.add(WelcomeScreenRoutineEnd());
const trialsExperimentBlockLoopScheduler = new Scheduler(psychoJS);
flowScheduler.add(trialsExperimentBlockLoopBegin(trialsExperimentBlockLoopScheduler));
flowScheduler.add(trialsExperimentBlockLoopScheduler);
flowScheduler.add(trialsExperimentBlockLoopEnd);
flowScheduler.add(ExperimentEndRoutineBegin());
flowScheduler.add(ExperimentEndRoutineEachFrame());
flowScheduler.add(ExperimentEndRoutineEnd());
flowScheduler.add(quitPsychoJS, '', true);

// quit if user presses Cancel in dialog box:
dialogCancelScheduler.add(quitPsychoJS, '', false);

psychoJS.start({
  expName: expName,
  expInfo: expInfo,
  resources: [
    {'name': 'resources/img/j_key.png', 'path': 'resources/img/j_key.png'},
    {'name': 'resources/img/thumbs_down.png', 'path': 'resources/img/thumbs_down.png'},
    {'name': 'resources/img/f_key.png', 'path': 'resources/img/f_key.png'},
    {'name': 'resources/img/thumbs_up.png', 'path': 'resources/img/thumbs_up.png'}
  ]
});

psychoJS.experimentLogger.setLevel(core.Logger.ServerLevel.EXP);


var frameDur;
async function updateInfo() {
  expInfo['date'] = util.MonotonicClock.getDateStr();  // add a simple timestamp
  expInfo['expName'] = expName;
  expInfo['psychopyVersion'] = '2022.1.3';
  expInfo['OS'] = window.navigator.platform;

  psychoJS.experiment.dataFileName = (("." + "/") + `data/${expInfo["participant"]}_${expName}_${expInfo["date"]}`);

  // store frame rate of monitor if we can measure it successfully
  expInfo['frameRate'] = psychoJS.window.getActualFrameRate();
  if (typeof expInfo['frameRate'] !== 'undefined')
    frameDur = 1.0 / Math.round(expInfo['frameRate']);
  else
    frameDur = 1.0 / 60.0; // couldn't get a reliable measure so guess

  // add info from the URL:
  util.addInfoFromUrl(expInfo);
  
  return Scheduler.Event.NEXT;
}


var ExperimentInitClock;
var WelcomeScreenClock;
var textWelcome;
var keyWelcone;
var blockWordLoaderClock;
var EncodingInstructClock;
var textEncodingInstruct;
var keyEncodingInstruct;
var FixationClock;
var textFixation;
var jitter;
var EncodingTrialClock;
var textEncodingTrial;
var key_respEncoding;
var EncodingEndClock;
var textEncodingEnd;
var keyEncodingEnd;
var PreTestCodeClock;
var TestingInstructClock;
var textTestingInstruct;
var keyTestingInstruct;
var imageFKey;
var imageJKey;
var textTestingInstructSpace;
var textTestingInstructF;
var textTestingInstructJ;
var TestInitCodeClock;
var TestingTrialClock;
var textTestingTrial;
var key_respTrial;
var CorrectnessSelectClock;
var nRepsCorrect;
var nRepsIncorrect;
var ConfidenceTrialClock;
var textConfidenceTrial;
var textTestingConfidenceF;
var textTestingConfidenceJ;
var key_respConfidence;
var TestingCorrectClock;
var textTestingCorrect;
var imageTestingCorrect;
var TestingIncorrectClock;
var textTestingIncorrect;
var imageTestingIncorrect;
var TestEndCodeClock;
var BlockSelectClock;
var TestingBlockEndClock;
var textTestBlockEnd;
var keyTestBlockEnd;
var ExperimentEndClock;
var textExperimentEnd;
var keyExperimentEnd;
var globalClock;
var routineTimer;
async function experimentInit() {
  // Initialize components for Routine "ExperimentInit"
  ExperimentInitClock = new util.Clock();
  // Initialize components for Routine "WelcomeScreen"
  WelcomeScreenClock = new util.Clock();
  textWelcome = new visual.TextStim({
    win: psychoJS.window,
    name: 'textWelcome',
    text: 'Welcome to the experiment.\n\nPlease press space to contiue.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyWelcone = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "blockWordLoader"
  blockWordLoaderClock = new util.Clock();
  // Initialize components for Routine "EncodingInstruct"
  EncodingInstructClock = new util.Clock();
  textEncodingInstruct = new visual.TextStim({
    win: psychoJS.window,
    name: 'textEncodingInstruct',
    text: 'In this phase of the experiments, you will see a series of words.\n\nDo your best to remember them.\n\nPress space to continue.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyEncodingInstruct = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "Fixation"
  FixationClock = new util.Clock();
  textFixation = new visual.TextStim({
    win: psychoJS.window,
    name: 'textFixation',
    text: '+',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  const round = function(num, n=0) {    
      return +(Math.round(num + ("e+" + n))  + ("e-" + n));
  }
  
  const randint = function(min, maxplusone) {
    return Math.floor(Math.random() * (maxplusone - min) ) + min;
  }
  
  jitter = (randint(0,301)+1000)/1000
  // Initialize components for Routine "EncodingTrial"
  EncodingTrialClock = new util.Clock();
  textEncodingTrial = new visual.TextStim({
    win: psychoJS.window,
    name: 'textEncodingTrial',
    text: '',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  key_respEncoding = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "EncodingEnd"
  EncodingEndClock = new util.Clock();
  textEncodingEnd = new visual.TextStim({
    win: psychoJS.window,
    name: 'textEncodingEnd',
    text: "Congratulations, you have completed this phase of the experiment.\n\nWhen you're ready, press space to move on to the next phase",
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyEncodingEnd = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "PreTestCode"
  PreTestCodeClock = new util.Clock();
  // Initialize components for Routine "TestingInstruct"
  TestingInstructClock = new util.Clock();
  textTestingInstruct = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingInstruct',
    text: 'In this phase of the experiment, you will see a mix of new and previously seen words.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.2], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyTestingInstruct = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  imageFKey = new visual.ImageStim({
    win : psychoJS.window,
    name : 'imageFKey', units : undefined, 
    image : 'resources/img/f_key.png', mask : undefined,
    ori : 0.0, pos : [(- 0.4), 0], size : [0.1, 0.1],
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -2.0 
  });
  imageJKey = new visual.ImageStim({
    win : psychoJS.window,
    name : 'imageJKey', units : undefined, 
    image : 'resources/img/j_key.png', mask : undefined,
    ori : 0.0, pos : [0.4, 0], size : [0.1, 0.1],
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -3.0 
  });
  textTestingInstructSpace = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingInstructSpace',
    text: 'Press space to continue.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, (- 0.3)], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -4.0 
  });
  
  textTestingInstructF = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingInstructF',
    text: 'Press F if you have not seen the word.',
    font: 'Open Sans',
    units: undefined, 
    pos: [(- 0.4), (- 0.1)], height: 0.05,  wrapWidth: 0.5, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -5.0 
  });
  
  textTestingInstructJ = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingInstructJ',
    text: 'Press J if you have seen the word.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0.4, (- 0.1)], height: 0.05,  wrapWidth: 0.5, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -6.0 
  });
  
  // Initialize components for Routine "TestInitCode"
  TestInitCodeClock = new util.Clock();
  // Initialize components for Routine "TestingTrial"
  TestingTrialClock = new util.Clock();
  textTestingTrial = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingTrial',
    text: '',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  key_respTrial = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "CorrectnessSelect"
  CorrectnessSelectClock = new util.Clock();
  nRepsCorrect = 0;
  nRepsIncorrect = 0;
  
  // Initialize components for Routine "ConfidenceTrial"
  ConfidenceTrialClock = new util.Clock();
  textConfidenceTrial = new visual.TextStim({
    win: psychoJS.window,
    name: 'textConfidenceTrial',
    text: 'What is your confidence level?',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.2], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  textTestingConfidenceF = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingConfidenceF',
    text: 'Low',
    font: 'Open Sans',
    units: undefined, 
    pos: [(- 0.4), (- 0.1)], height: 0.05,  wrapWidth: 0.5, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -1.0 
  });
  
  textTestingConfidenceJ = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingConfidenceJ',
    text: 'High',
    font: 'Open Sans',
    units: undefined, 
    pos: [0.4, (- 0.1)], height: 0.05,  wrapWidth: 0.5, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: -2.0 
  });
  
  key_respConfidence = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "TestingCorrect"
  TestingCorrectClock = new util.Clock();
  textTestingCorrect = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingCorrect',
    text: 'Your answer was:',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.15], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  imageTestingCorrect = new visual.ImageStim({
    win : psychoJS.window,
    name : 'imageTestingCorrect', units : undefined, 
    image : 'resources/img/thumbs_up.png', mask : undefined,
    ori : 0.0, pos : [0, (- 0.05)], size : [0.15, 0.15],
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -1.0 
  });
  // Initialize components for Routine "TestingIncorrect"
  TestingIncorrectClock = new util.Clock();
  textTestingIncorrect = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestingIncorrect',
    text: 'Your answer was',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0.15], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  imageTestingIncorrect = new visual.ImageStim({
    win : psychoJS.window,
    name : 'imageTestingIncorrect', units : undefined, 
    image : 'resources/img/thumbs_down.png', mask : undefined,
    ori : 0.0, pos : [0, (- 0.05)], size : [0.15, 0.15],
    color : new util.Color([1,1,1]), opacity : undefined,
    flipHoriz : false, flipVert : false,
    texRes : 128.0, interpolate : true, depth : -1.0 
  });
  // Initialize components for Routine "TestEndCode"
  TestEndCodeClock = new util.Clock();
  // Initialize components for Routine "BlockSelect"
  BlockSelectClock = new util.Clock();
  // Initialize components for Routine "TestingBlockEnd"
  TestingBlockEndClock = new util.Clock();
  textTestBlockEnd = new visual.TextStim({
    win: psychoJS.window,
    name: 'textTestBlockEnd',
    text: "Congratulations, you have completed this phase of the experiment.\n\nWhen you're ready, press space to move on to the next phase",
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyTestBlockEnd = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Initialize components for Routine "ExperimentEnd"
  ExperimentEndClock = new util.Clock();
  textExperimentEnd = new visual.TextStim({
    win: psychoJS.window,
    name: 'textExperimentEnd',
    text: 'Thank you, the experiment is now over.',
    font: 'Open Sans',
    units: undefined, 
    pos: [0, 0], height: 0.05,  wrapWidth: undefined, ori: 0.0,
    color: new util.Color('white'),  opacity: undefined,
    depth: 0.0 
  });
  
  keyExperimentEnd = new core.Keyboard({psychoJS: psychoJS, clock: new util.Clock(), waitForStart: true});
  
  // Create some handy timers
  globalClock = new util.Clock();  // to track the time since experiment started
  routineTimer = new util.CountdownTimer();  // to track time remaining of each (non-slip) routine
  
  return Scheduler.Event.NEXT;
}


var t;
var frameN;
var continueRoutine;
var ExperimentInitComponents;
function ExperimentInitRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'ExperimentInit'-------
    t = 0;
    ExperimentInitClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    ExperimentInitComponents = [];
    
    for (const thisComponent of ExperimentInitComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function ExperimentInitRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'ExperimentInit'-------
    // get current time
    t = ExperimentInitClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of ExperimentInitComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function ExperimentInitRoutineEnd() {
  return async function () {
    //------Ending Routine 'ExperimentInit'-------
    for (const thisComponent of ExperimentInitComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "ExperimentInit" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyWelcone_allKeys;
var WelcomeScreenComponents;
function WelcomeScreenRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'WelcomeScreen'-------
    t = 0;
    WelcomeScreenClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyWelcone.keys = undefined;
    keyWelcone.rt = undefined;
    _keyWelcone_allKeys = [];
    // keep track of which components have finished
    WelcomeScreenComponents = [];
    WelcomeScreenComponents.push(textWelcome);
    WelcomeScreenComponents.push(keyWelcone);
    
    for (const thisComponent of WelcomeScreenComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function WelcomeScreenRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'WelcomeScreen'-------
    // get current time
    t = WelcomeScreenClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textWelcome* updates
    if (t >= 0.0 && textWelcome.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textWelcome.tStart = t;  // (not accounting for frame time here)
      textWelcome.frameNStart = frameN;  // exact frame index
      
      textWelcome.setAutoDraw(true);
    }

    
    // *keyWelcone* updates
    if (t >= 0.0 && keyWelcone.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyWelcone.tStart = t;  // (not accounting for frame time here)
      keyWelcone.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyWelcone.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyWelcone.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyWelcone.clearEvents(); });
    }

    if (keyWelcone.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyWelcone.getKeys({keyList: ['space'], waitRelease: false});
      _keyWelcone_allKeys = _keyWelcone_allKeys.concat(theseKeys);
      if (_keyWelcone_allKeys.length > 0) {
        keyWelcone.keys = _keyWelcone_allKeys[_keyWelcone_allKeys.length - 1].name;  // just the last key pressed
        keyWelcone.rt = _keyWelcone_allKeys[_keyWelcone_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of WelcomeScreenComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function WelcomeScreenRoutineEnd() {
  return async function () {
    //------Ending Routine 'WelcomeScreen'-------
    for (const thisComponent of WelcomeScreenComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyWelcone.stop();
    // the Routine "WelcomeScreen" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var trialsExperimentBlock;
var currentLoop;
function trialsExperimentBlockLoopBegin(trialsExperimentBlockLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    trialsExperimentBlock = new TrialHandler({
      psychoJS: psychoJS,
      nReps: numberOfBlocks, method: TrialHandler.Method.SEQUENTIAL,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'trialsExperimentBlock'
    });
    psychoJS.experiment.addLoop(trialsExperimentBlock); // add the loop to the experiment
    currentLoop = trialsExperimentBlock;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTrialsExperimentBlock of trialsExperimentBlock) {
      const snapshot = trialsExperimentBlock.getSnapshot();
      trialsExperimentBlockLoopScheduler.add(importConditions(snapshot));
      trialsExperimentBlockLoopScheduler.add(blockWordLoaderRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(blockWordLoaderRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(blockWordLoaderRoutineEnd());
      trialsExperimentBlockLoopScheduler.add(EncodingInstructRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(EncodingInstructRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(EncodingInstructRoutineEnd());
      const trialsEncodingBlockLoopScheduler = new Scheduler(psychoJS);
      trialsExperimentBlockLoopScheduler.add(trialsEncodingBlockLoopBegin(trialsEncodingBlockLoopScheduler, snapshot));
      trialsExperimentBlockLoopScheduler.add(trialsEncodingBlockLoopScheduler);
      trialsExperimentBlockLoopScheduler.add(trialsEncodingBlockLoopEnd);
      trialsExperimentBlockLoopScheduler.add(EncodingEndRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(EncodingEndRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(EncodingEndRoutineEnd());
      trialsExperimentBlockLoopScheduler.add(PreTestCodeRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(PreTestCodeRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(PreTestCodeRoutineEnd());
      trialsExperimentBlockLoopScheduler.add(TestingInstructRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(TestingInstructRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(TestingInstructRoutineEnd());
      const trialsTestingBlockLoopScheduler = new Scheduler(psychoJS);
      trialsExperimentBlockLoopScheduler.add(trialsTestingBlockLoopBegin(trialsTestingBlockLoopScheduler, snapshot));
      trialsExperimentBlockLoopScheduler.add(trialsTestingBlockLoopScheduler);
      trialsExperimentBlockLoopScheduler.add(trialsTestingBlockLoopEnd);
      trialsExperimentBlockLoopScheduler.add(BlockSelectRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(BlockSelectRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(BlockSelectRoutineEnd());
      trialsExperimentBlockLoopScheduler.add(TestingBlockEndRoutineBegin(snapshot));
      trialsExperimentBlockLoopScheduler.add(TestingBlockEndRoutineEachFrame());
      trialsExperimentBlockLoopScheduler.add(TestingBlockEndRoutineEnd());
      trialsExperimentBlockLoopScheduler.add(endLoopIteration(trialsExperimentBlockLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


var trialsEncodingBlock;
function trialsEncodingBlockLoopBegin(trialsEncodingBlockLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    trialsEncodingBlock = new TrialHandler({
      psychoJS: psychoJS,
      nReps: currentDifficultyLevel.numEncoding, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'trialsEncodingBlock'
    });
    psychoJS.experiment.addLoop(trialsEncodingBlock); // add the loop to the experiment
    currentLoop = trialsEncodingBlock;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTrialsEncodingBlock of trialsEncodingBlock) {
      const snapshot = trialsEncodingBlock.getSnapshot();
      trialsEncodingBlockLoopScheduler.add(importConditions(snapshot));
      trialsEncodingBlockLoopScheduler.add(FixationRoutineBegin(snapshot));
      trialsEncodingBlockLoopScheduler.add(FixationRoutineEachFrame());
      trialsEncodingBlockLoopScheduler.add(FixationRoutineEnd());
      trialsEncodingBlockLoopScheduler.add(EncodingTrialRoutineBegin(snapshot));
      trialsEncodingBlockLoopScheduler.add(EncodingTrialRoutineEachFrame());
      trialsEncodingBlockLoopScheduler.add(EncodingTrialRoutineEnd());
      trialsEncodingBlockLoopScheduler.add(endLoopIteration(trialsEncodingBlockLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


async function trialsEncodingBlockLoopEnd() {
  psychoJS.experiment.removeLoop(trialsEncodingBlock);

  return Scheduler.Event.NEXT;
}


var trialsTestingBlock;
function trialsTestingBlockLoopBegin(trialsTestingBlockLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    trialsTestingBlock = new TrialHandler({
      psychoJS: psychoJS,
      nReps: numberTestTrials, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'trialsTestingBlock'
    });
    psychoJS.experiment.addLoop(trialsTestingBlock); // add the loop to the experiment
    currentLoop = trialsTestingBlock;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTrialsTestingBlock of trialsTestingBlock) {
      const snapshot = trialsTestingBlock.getSnapshot();
      trialsTestingBlockLoopScheduler.add(importConditions(snapshot));
      trialsTestingBlockLoopScheduler.add(TestInitCodeRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(TestInitCodeRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(TestInitCodeRoutineEnd());
      trialsTestingBlockLoopScheduler.add(FixationRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(FixationRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(FixationRoutineEnd());
      trialsTestingBlockLoopScheduler.add(TestingTrialRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(TestingTrialRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(TestingTrialRoutineEnd());
      trialsTestingBlockLoopScheduler.add(CorrectnessSelectRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(CorrectnessSelectRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(CorrectnessSelectRoutineEnd());
      trialsTestingBlockLoopScheduler.add(FixationRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(FixationRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(FixationRoutineEnd());
      trialsTestingBlockLoopScheduler.add(ConfidenceTrialRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(ConfidenceTrialRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(ConfidenceTrialRoutineEnd());
      trialsTestingBlockLoopScheduler.add(FixationRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(FixationRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(FixationRoutineEnd());
      const trialsTestingCorrectLoopScheduler = new Scheduler(psychoJS);
      trialsTestingBlockLoopScheduler.add(trialsTestingCorrectLoopBegin(trialsTestingCorrectLoopScheduler, snapshot));
      trialsTestingBlockLoopScheduler.add(trialsTestingCorrectLoopScheduler);
      trialsTestingBlockLoopScheduler.add(trialsTestingCorrectLoopEnd);
      const trialsTestingIncorrectLoopScheduler = new Scheduler(psychoJS);
      trialsTestingBlockLoopScheduler.add(trialsTestingIncorrectLoopBegin(trialsTestingIncorrectLoopScheduler, snapshot));
      trialsTestingBlockLoopScheduler.add(trialsTestingIncorrectLoopScheduler);
      trialsTestingBlockLoopScheduler.add(trialsTestingIncorrectLoopEnd);
      trialsTestingBlockLoopScheduler.add(TestEndCodeRoutineBegin(snapshot));
      trialsTestingBlockLoopScheduler.add(TestEndCodeRoutineEachFrame());
      trialsTestingBlockLoopScheduler.add(TestEndCodeRoutineEnd());
      trialsTestingBlockLoopScheduler.add(endLoopIteration(trialsTestingBlockLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


var trialsTestingCorrect;
function trialsTestingCorrectLoopBegin(trialsTestingCorrectLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    trialsTestingCorrect = new TrialHandler({
      psychoJS: psychoJS,
      nReps: nRepsCorrect, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'trialsTestingCorrect'
    });
    psychoJS.experiment.addLoop(trialsTestingCorrect); // add the loop to the experiment
    currentLoop = trialsTestingCorrect;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTrialsTestingCorrect of trialsTestingCorrect) {
      const snapshot = trialsTestingCorrect.getSnapshot();
      trialsTestingCorrectLoopScheduler.add(importConditions(snapshot));
      trialsTestingCorrectLoopScheduler.add(TestingCorrectRoutineBegin(snapshot));
      trialsTestingCorrectLoopScheduler.add(TestingCorrectRoutineEachFrame());
      trialsTestingCorrectLoopScheduler.add(TestingCorrectRoutineEnd());
      trialsTestingCorrectLoopScheduler.add(endLoopIteration(trialsTestingCorrectLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


async function trialsTestingCorrectLoopEnd() {
  psychoJS.experiment.removeLoop(trialsTestingCorrect);

  return Scheduler.Event.NEXT;
}


var trialsTestingIncorrect;
function trialsTestingIncorrectLoopBegin(trialsTestingIncorrectLoopScheduler, snapshot) {
  return async function() {
    TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop
    
    // set up handler to look after randomisation of conditions etc
    trialsTestingIncorrect = new TrialHandler({
      psychoJS: psychoJS,
      nReps: nRepsIncorrect, method: TrialHandler.Method.RANDOM,
      extraInfo: expInfo, originPath: undefined,
      trialList: undefined,
      seed: undefined, name: 'trialsTestingIncorrect'
    });
    psychoJS.experiment.addLoop(trialsTestingIncorrect); // add the loop to the experiment
    currentLoop = trialsTestingIncorrect;  // we're now the current loop
    
    // Schedule all the trials in the trialList:
    for (const thisTrialsTestingIncorrect of trialsTestingIncorrect) {
      const snapshot = trialsTestingIncorrect.getSnapshot();
      trialsTestingIncorrectLoopScheduler.add(importConditions(snapshot));
      trialsTestingIncorrectLoopScheduler.add(TestingIncorrectRoutineBegin(snapshot));
      trialsTestingIncorrectLoopScheduler.add(TestingIncorrectRoutineEachFrame());
      trialsTestingIncorrectLoopScheduler.add(TestingIncorrectRoutineEnd());
      trialsTestingIncorrectLoopScheduler.add(endLoopIteration(trialsTestingIncorrectLoopScheduler, snapshot));
    }
    
    return Scheduler.Event.NEXT;
  }
}


async function trialsTestingIncorrectLoopEnd() {
  psychoJS.experiment.removeLoop(trialsTestingIncorrect);

  return Scheduler.Event.NEXT;
}


async function trialsTestingBlockLoopEnd() {
  psychoJS.experiment.removeLoop(trialsTestingBlock);

  return Scheduler.Event.NEXT;
}


async function trialsExperimentBlockLoopEnd() {
  psychoJS.experiment.removeLoop(trialsExperimentBlock);

  return Scheduler.Event.NEXT;
}


var blockWordLoaderComponents;
function blockWordLoaderRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'blockWordLoader'-------
    t = 0;
    blockWordLoaderClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    blockWordLoaderComponents = [];
    
    for (const thisComponent of blockWordLoaderComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function blockWordLoaderRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'blockWordLoader'-------
    // get current time
    t = blockWordLoaderClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of blockWordLoaderComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function blockWordLoaderRoutineEnd() {
  return async function () {
    //------Ending Routine 'blockWordLoader'-------
    for (const thisComponent of blockWordLoaderComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "blockWordLoader" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyEncodingInstruct_allKeys;
var EncodingInstructComponents;
function EncodingInstructRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'EncodingInstruct'-------
    t = 0;
    EncodingInstructClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyEncodingInstruct.keys = undefined;
    keyEncodingInstruct.rt = undefined;
    _keyEncodingInstruct_allKeys = [];
    // keep track of which components have finished
    EncodingInstructComponents = [];
    EncodingInstructComponents.push(textEncodingInstruct);
    EncodingInstructComponents.push(keyEncodingInstruct);
    
    for (const thisComponent of EncodingInstructComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function EncodingInstructRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'EncodingInstruct'-------
    // get current time
    t = EncodingInstructClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textEncodingInstruct* updates
    if (t >= 0.0 && textEncodingInstruct.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textEncodingInstruct.tStart = t;  // (not accounting for frame time here)
      textEncodingInstruct.frameNStart = frameN;  // exact frame index
      
      textEncodingInstruct.setAutoDraw(true);
    }

    
    // *keyEncodingInstruct* updates
    if (t >= 0.0 && keyEncodingInstruct.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyEncodingInstruct.tStart = t;  // (not accounting for frame time here)
      keyEncodingInstruct.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyEncodingInstruct.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyEncodingInstruct.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyEncodingInstruct.clearEvents(); });
    }

    if (keyEncodingInstruct.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyEncodingInstruct.getKeys({keyList: ['space'], waitRelease: false});
      _keyEncodingInstruct_allKeys = _keyEncodingInstruct_allKeys.concat(theseKeys);
      if (_keyEncodingInstruct_allKeys.length > 0) {
        keyEncodingInstruct.keys = _keyEncodingInstruct_allKeys[_keyEncodingInstruct_allKeys.length - 1].name;  // just the last key pressed
        keyEncodingInstruct.rt = _keyEncodingInstruct_allKeys[_keyEncodingInstruct_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of EncodingInstructComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function EncodingInstructRoutineEnd() {
  return async function () {
    //------Ending Routine 'EncodingInstruct'-------
    for (const thisComponent of EncodingInstructComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyEncodingInstruct.stop();
    // the Routine "EncodingInstruct" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var FixationComponents;
function FixationRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'Fixation'-------
    t = 0;
    FixationClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    const round = function(num, n=0) {    
        return +(Math.round(num + ("e+" + n))  + ("e-" + n));
    }
    
    const randint = function(min, maxplusone) {
      return Math.floor(Math.random() * (maxplusone - min) ) + min;
    }
    
    jitter = (randint(0,301)+1000)/1000
    // keep track of which components have finished
    FixationComponents = [];
    FixationComponents.push(textFixation);
    
    for (const thisComponent of FixationComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


var frameRemains;
function FixationRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'Fixation'-------
    // get current time
    t = FixationClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textFixation* updates
    if (t >= 0.0 && textFixation.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textFixation.tStart = t;  // (not accounting for frame time here)
      textFixation.frameNStart = frameN;  // exact frame index
      
      textFixation.setAutoDraw(true);
    }

    frameRemains = 0.0 + jitter - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textFixation.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textFixation.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of FixationComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function FixationRoutineEnd() {
  return async function () {
    //------Ending Routine 'Fixation'-------
    for (const thisComponent of FixationComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "Fixation" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _key_respEncoding_allKeys;
var EncodingTrialComponents;
function EncodingTrialRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'EncodingTrial'-------
    t = 0;
    EncodingTrialClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    textEncodingTrial.setText(encodingWordItem[currentEncodingWord]);
    key_respEncoding.keys = undefined;
    key_respEncoding.rt = undefined;
    _key_respEncoding_allKeys = [];
    // keep track of which components have finished
    EncodingTrialComponents = [];
    EncodingTrialComponents.push(textEncodingTrial);
    EncodingTrialComponents.push(key_respEncoding);
    
    for (const thisComponent of EncodingTrialComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function EncodingTrialRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'EncodingTrial'-------
    // get current time
    t = EncodingTrialClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textEncodingTrial* updates
    if (t >= 0.0 && textEncodingTrial.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textEncodingTrial.tStart = t;  // (not accounting for frame time here)
      textEncodingTrial.frameNStart = frameN;  // exact frame index
      
      textEncodingTrial.setAutoDraw(true);
    }

    frameRemains = 0.0 + currentDifficultyLevel.duration - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textEncodingTrial.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textEncodingTrial.setAutoDraw(false);
    }
    
    // *key_respEncoding* updates
    if (t >= 0.0 && key_respEncoding.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_respEncoding.tStart = t;  // (not accounting for frame time here)
      key_respEncoding.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_respEncoding.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_respEncoding.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_respEncoding.clearEvents(); });
    }

    frameRemains = 0.0 + currentDifficultyLevel.duration - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (key_respEncoding.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      key_respEncoding.status = PsychoJS.Status.FINISHED;
  }

    if (key_respEncoding.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_respEncoding.getKeys({keyList: ['q'], waitRelease: false});
      _key_respEncoding_allKeys = _key_respEncoding_allKeys.concat(theseKeys);
      if (_key_respEncoding_allKeys.length > 0) {
        key_respEncoding.keys = _key_respEncoding_allKeys[_key_respEncoding_allKeys.length - 1].name;  // just the last key pressed
        key_respEncoding.rt = _key_respEncoding_allKeys[_key_respEncoding_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of EncodingTrialComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function EncodingTrialRoutineEnd() {
  return async function () {
    //------Ending Routine 'EncodingTrial'-------
    for (const thisComponent of EncodingTrialComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    key_respEncoding.stop();
    if (((key_respEncoding.keys !== undefined) && (key_respEncoding.keys === "q"))) {
        trialsEncoding.finished = 1;
    }
    currentEncodingWord += 1;
    
    // the Routine "EncodingTrial" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyEncodingEnd_allKeys;
var EncodingEndComponents;
function EncodingEndRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'EncodingEnd'-------
    t = 0;
    EncodingEndClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyEncodingEnd.keys = undefined;
    keyEncodingEnd.rt = undefined;
    _keyEncodingEnd_allKeys = [];
    // keep track of which components have finished
    EncodingEndComponents = [];
    EncodingEndComponents.push(textEncodingEnd);
    EncodingEndComponents.push(keyEncodingEnd);
    
    for (const thisComponent of EncodingEndComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function EncodingEndRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'EncodingEnd'-------
    // get current time
    t = EncodingEndClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textEncodingEnd* updates
    if (t >= 0.0 && textEncodingEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textEncodingEnd.tStart = t;  // (not accounting for frame time here)
      textEncodingEnd.frameNStart = frameN;  // exact frame index
      
      textEncodingEnd.setAutoDraw(true);
    }

    
    // *keyEncodingEnd* updates
    if (t >= 0.0 && keyEncodingEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyEncodingEnd.tStart = t;  // (not accounting for frame time here)
      keyEncodingEnd.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyEncodingEnd.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyEncodingEnd.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyEncodingEnd.clearEvents(); });
    }

    if (keyEncodingEnd.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyEncodingEnd.getKeys({keyList: ['space'], waitRelease: false});
      _keyEncodingEnd_allKeys = _keyEncodingEnd_allKeys.concat(theseKeys);
      if (_keyEncodingEnd_allKeys.length > 0) {
        keyEncodingEnd.keys = _keyEncodingEnd_allKeys[_keyEncodingEnd_allKeys.length - 1].name;  // just the last key pressed
        keyEncodingEnd.rt = _keyEncodingEnd_allKeys[_keyEncodingEnd_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of EncodingEndComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function EncodingEndRoutineEnd() {
  return async function () {
    //------Ending Routine 'EncodingEnd'-------
    for (const thisComponent of EncodingEndComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyEncodingEnd.stop();
    // the Routine "EncodingEnd" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var PreTestCodeComponents;
function PreTestCodeRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'PreTestCode'-------
    t = 0;
    PreTestCodeClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    PreTestCodeComponents = [];
    
    for (const thisComponent of PreTestCodeComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function PreTestCodeRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'PreTestCode'-------
    // get current time
    t = PreTestCodeClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of PreTestCodeComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function PreTestCodeRoutineEnd() {
  return async function () {
    //------Ending Routine 'PreTestCode'-------
    for (const thisComponent of PreTestCodeComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "PreTestCode" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyTestingInstruct_allKeys;
var TestingInstructComponents;
function TestingInstructRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestingInstruct'-------
    t = 0;
    TestingInstructClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyTestingInstruct.keys = undefined;
    keyTestingInstruct.rt = undefined;
    _keyTestingInstruct_allKeys = [];
    // keep track of which components have finished
    TestingInstructComponents = [];
    TestingInstructComponents.push(textTestingInstruct);
    TestingInstructComponents.push(keyTestingInstruct);
    TestingInstructComponents.push(imageFKey);
    TestingInstructComponents.push(imageJKey);
    TestingInstructComponents.push(textTestingInstructSpace);
    TestingInstructComponents.push(textTestingInstructF);
    TestingInstructComponents.push(textTestingInstructJ);
    
    for (const thisComponent of TestingInstructComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestingInstructRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestingInstruct'-------
    // get current time
    t = TestingInstructClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textTestingInstruct* updates
    if (t >= 0.0 && textTestingInstruct.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingInstruct.tStart = t;  // (not accounting for frame time here)
      textTestingInstruct.frameNStart = frameN;  // exact frame index
      
      textTestingInstruct.setAutoDraw(true);
    }

    
    // *keyTestingInstruct* updates
    if (t >= 0.0 && keyTestingInstruct.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyTestingInstruct.tStart = t;  // (not accounting for frame time here)
      keyTestingInstruct.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyTestingInstruct.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyTestingInstruct.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyTestingInstruct.clearEvents(); });
    }

    if (keyTestingInstruct.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyTestingInstruct.getKeys({keyList: ['space'], waitRelease: false});
      _keyTestingInstruct_allKeys = _keyTestingInstruct_allKeys.concat(theseKeys);
      if (_keyTestingInstruct_allKeys.length > 0) {
        keyTestingInstruct.keys = _keyTestingInstruct_allKeys[_keyTestingInstruct_allKeys.length - 1].name;  // just the last key pressed
        keyTestingInstruct.rt = _keyTestingInstruct_allKeys[_keyTestingInstruct_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    
    // *imageFKey* updates
    if (t >= 0.0 && imageFKey.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      imageFKey.tStart = t;  // (not accounting for frame time here)
      imageFKey.frameNStart = frameN;  // exact frame index
      
      imageFKey.setAutoDraw(true);
    }

    
    // *imageJKey* updates
    if (t >= 0.0 && imageJKey.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      imageJKey.tStart = t;  // (not accounting for frame time here)
      imageJKey.frameNStart = frameN;  // exact frame index
      
      imageJKey.setAutoDraw(true);
    }

    
    // *textTestingInstructSpace* updates
    if (t >= 0.0 && textTestingInstructSpace.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingInstructSpace.tStart = t;  // (not accounting for frame time here)
      textTestingInstructSpace.frameNStart = frameN;  // exact frame index
      
      textTestingInstructSpace.setAutoDraw(true);
    }

    
    // *textTestingInstructF* updates
    if (t >= 0.0 && textTestingInstructF.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingInstructF.tStart = t;  // (not accounting for frame time here)
      textTestingInstructF.frameNStart = frameN;  // exact frame index
      
      textTestingInstructF.setAutoDraw(true);
    }

    
    // *textTestingInstructJ* updates
    if (t >= 0.0 && textTestingInstructJ.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingInstructJ.tStart = t;  // (not accounting for frame time here)
      textTestingInstructJ.frameNStart = frameN;  // exact frame index
      
      textTestingInstructJ.setAutoDraw(true);
    }

    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestingInstructComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestingInstructRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestingInstruct'-------
    for (const thisComponent of TestingInstructComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyTestingInstruct.stop();
    // the Routine "TestingInstruct" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var TestInitCodeComponents;
function TestInitCodeRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestInitCode'-------
    t = 0;
    TestInitCodeClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    TestInitCodeComponents = [];
    
    for (const thisComponent of TestInitCodeComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestInitCodeRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestInitCode'-------
    // get current time
    t = TestInitCodeClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestInitCodeComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestInitCodeRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestInitCode'-------
    for (const thisComponent of TestInitCodeComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "TestInitCode" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _key_respTrial_allKeys;
var TestingTrialComponents;
function TestingTrialRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestingTrial'-------
    t = 0;
    TestingTrialClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    routineTimer.add(1.300000);
    // update component parameters for each repeat
    textTestingTrial.setText(testWord);
    key_respTrial.keys = undefined;
    key_respTrial.rt = undefined;
    _key_respTrial_allKeys = [];
    // keep track of which components have finished
    TestingTrialComponents = [];
    TestingTrialComponents.push(textTestingTrial);
    TestingTrialComponents.push(key_respTrial);
    
    for (const thisComponent of TestingTrialComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestingTrialRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestingTrial'-------
    // get current time
    t = TestingTrialClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textTestingTrial* updates
    if (t >= 0.0 && textTestingTrial.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingTrial.tStart = t;  // (not accounting for frame time here)
      textTestingTrial.frameNStart = frameN;  // exact frame index
      
      textTestingTrial.setAutoDraw(true);
    }

    frameRemains = 0.0 + 1.3 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textTestingTrial.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textTestingTrial.setAutoDraw(false);
    }
    
    // *key_respTrial* updates
    if (t >= 0.0 && key_respTrial.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_respTrial.tStart = t;  // (not accounting for frame time here)
      key_respTrial.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_respTrial.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_respTrial.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_respTrial.clearEvents(); });
    }

    frameRemains = 0.0 + 1.3 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (key_respTrial.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      key_respTrial.status = PsychoJS.Status.FINISHED;
  }

    if (key_respTrial.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_respTrial.getKeys({keyList: ['f', 'j', 'q'], waitRelease: false});
      _key_respTrial_allKeys = _key_respTrial_allKeys.concat(theseKeys);
      if (_key_respTrial_allKeys.length > 0) {
        key_respTrial.keys = _key_respTrial_allKeys[0].name;  // just the first key pressed
        key_respTrial.rt = _key_respTrial_allKeys[0].rt;
        // was this correct?
        if (key_respTrial.keys == testCorrect) {
            key_respTrial.corr = 1;
        } else {
            key_respTrial.corr = 0;
        }
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestingTrialComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestingTrialRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestingTrial'-------
    for (const thisComponent of TestingTrialComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // was no response the correct answer?!
    if (key_respTrial.keys === undefined) {
      if (['None','none',undefined].includes(testCorrect)) {
         key_respTrial.corr = 1;  // correct non-response
      } else {
         key_respTrial.corr = 0;  // failed to respond (incorrectly)
      }
    }
    // store data for psychoJS.experiment (ExperimentHandler)
    // update the trial handler
    if (currentLoop instanceof MultiStairHandler) {
      currentLoop.addResponse(key_respTrial.corr, level);
    }
    psychoJS.experiment.addData('key_respTrial.keys', key_respTrial.keys);
    psychoJS.experiment.addData('key_respTrial.corr', key_respTrial.corr);
    if (typeof key_respTrial.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_respTrial.rt', key_respTrial.rt);
        routineTimer.reset();
        }
    
    key_respTrial.stop();
    return Scheduler.Event.NEXT;
  };
}


var CorrectnessSelectComponents;
function CorrectnessSelectRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'CorrectnessSelect'-------
    t = 0;
    CorrectnessSelectClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    if ((key_respTrial.corr === 1)) {
        nRepsCorrect = 1;
        nRepsIncorrect = 0;
    } else {
        nRepsCorrect = 0;
        nRepsIncorrect = 1;
    }
    
    // keep track of which components have finished
    CorrectnessSelectComponents = [];
    
    for (const thisComponent of CorrectnessSelectComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function CorrectnessSelectRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'CorrectnessSelect'-------
    // get current time
    t = CorrectnessSelectClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of CorrectnessSelectComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function CorrectnessSelectRoutineEnd() {
  return async function () {
    //------Ending Routine 'CorrectnessSelect'-------
    for (const thisComponent of CorrectnessSelectComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "CorrectnessSelect" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _key_respConfidence_allKeys;
var ConfidenceTrialComponents;
function ConfidenceTrialRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'ConfidenceTrial'-------
    t = 0;
    ConfidenceTrialClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    routineTimer.add(1.500000);
    // update component parameters for each repeat
    key_respConfidence.keys = undefined;
    key_respConfidence.rt = undefined;
    _key_respConfidence_allKeys = [];
    // keep track of which components have finished
    ConfidenceTrialComponents = [];
    ConfidenceTrialComponents.push(textConfidenceTrial);
    ConfidenceTrialComponents.push(textTestingConfidenceF);
    ConfidenceTrialComponents.push(textTestingConfidenceJ);
    ConfidenceTrialComponents.push(key_respConfidence);
    
    for (const thisComponent of ConfidenceTrialComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function ConfidenceTrialRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'ConfidenceTrial'-------
    // get current time
    t = ConfidenceTrialClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textConfidenceTrial* updates
    if (t >= 0.0 && textConfidenceTrial.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textConfidenceTrial.tStart = t;  // (not accounting for frame time here)
      textConfidenceTrial.frameNStart = frameN;  // exact frame index
      
      textConfidenceTrial.setAutoDraw(true);
    }

    frameRemains = 0.0 + 1.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textConfidenceTrial.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textConfidenceTrial.setAutoDraw(false);
    }
    
    // *textTestingConfidenceF* updates
    if (t >= 0.0 && textTestingConfidenceF.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingConfidenceF.tStart = t;  // (not accounting for frame time here)
      textTestingConfidenceF.frameNStart = frameN;  // exact frame index
      
      textTestingConfidenceF.setAutoDraw(true);
    }

    frameRemains = 0.0 + 1.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textTestingConfidenceF.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textTestingConfidenceF.setAutoDraw(false);
    }
    
    // *textTestingConfidenceJ* updates
    if (t >= 0.0 && textTestingConfidenceJ.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingConfidenceJ.tStart = t;  // (not accounting for frame time here)
      textTestingConfidenceJ.frameNStart = frameN;  // exact frame index
      
      textTestingConfidenceJ.setAutoDraw(true);
    }

    frameRemains = 0.0 + 1.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textTestingConfidenceJ.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textTestingConfidenceJ.setAutoDraw(false);
    }
    
    // *key_respConfidence* updates
    if (t >= 0.0 && key_respConfidence.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      key_respConfidence.tStart = t;  // (not accounting for frame time here)
      key_respConfidence.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { key_respConfidence.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { key_respConfidence.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { key_respConfidence.clearEvents(); });
    }

    frameRemains = 0.0 + 1.5 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (key_respConfidence.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      key_respConfidence.status = PsychoJS.Status.FINISHED;
  }

    if (key_respConfidence.status === PsychoJS.Status.STARTED) {
      let theseKeys = key_respConfidence.getKeys({keyList: ['f', 'j'], waitRelease: false});
      _key_respConfidence_allKeys = _key_respConfidence_allKeys.concat(theseKeys);
      if (_key_respConfidence_allKeys.length > 0) {
        key_respConfidence.keys = _key_respConfidence_allKeys[_key_respConfidence_allKeys.length - 1].name;  // just the last key pressed
        key_respConfidence.rt = _key_respConfidence_allKeys[_key_respConfidence_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of ConfidenceTrialComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function ConfidenceTrialRoutineEnd() {
  return async function () {
    //------Ending Routine 'ConfidenceTrial'-------
    for (const thisComponent of ConfidenceTrialComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // update the trial handler
    if (currentLoop instanceof MultiStairHandler) {
      currentLoop.addResponse(key_respConfidence.corr, level);
    }
    psychoJS.experiment.addData('key_respConfidence.keys', key_respConfidence.keys);
    if (typeof key_respConfidence.keys !== 'undefined') {  // we had a response
        psychoJS.experiment.addData('key_respConfidence.rt', key_respConfidence.rt);
        routineTimer.reset();
        }
    
    key_respConfidence.stop();
    return Scheduler.Event.NEXT;
  };
}


var TestingCorrectComponents;
function TestingCorrectRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestingCorrect'-------
    t = 0;
    TestingCorrectClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    routineTimer.add(0.200000);
    // update component parameters for each repeat
    // keep track of which components have finished
    TestingCorrectComponents = [];
    TestingCorrectComponents.push(textTestingCorrect);
    TestingCorrectComponents.push(imageTestingCorrect);
    
    for (const thisComponent of TestingCorrectComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestingCorrectRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestingCorrect'-------
    // get current time
    t = TestingCorrectClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textTestingCorrect* updates
    if (t >= 0.0 && textTestingCorrect.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingCorrect.tStart = t;  // (not accounting for frame time here)
      textTestingCorrect.frameNStart = frameN;  // exact frame index
      
      textTestingCorrect.setAutoDraw(true);
    }

    frameRemains = 0.0 + 0.2 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textTestingCorrect.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textTestingCorrect.setAutoDraw(false);
    }
    
    // *imageTestingCorrect* updates
    if (t >= 0.0 && imageTestingCorrect.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      imageTestingCorrect.tStart = t;  // (not accounting for frame time here)
      imageTestingCorrect.frameNStart = frameN;  // exact frame index
      
      imageTestingCorrect.setAutoDraw(true);
    }

    frameRemains = 0.0 + 0.2 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (imageTestingCorrect.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      imageTestingCorrect.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestingCorrectComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestingCorrectRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestingCorrect'-------
    for (const thisComponent of TestingCorrectComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    return Scheduler.Event.NEXT;
  };
}


var TestingIncorrectComponents;
function TestingIncorrectRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestingIncorrect'-------
    t = 0;
    TestingIncorrectClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    routineTimer.add(0.200000);
    // update component parameters for each repeat
    // keep track of which components have finished
    TestingIncorrectComponents = [];
    TestingIncorrectComponents.push(textTestingIncorrect);
    TestingIncorrectComponents.push(imageTestingIncorrect);
    
    for (const thisComponent of TestingIncorrectComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestingIncorrectRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestingIncorrect'-------
    // get current time
    t = TestingIncorrectClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textTestingIncorrect* updates
    if (t >= 0.0 && textTestingIncorrect.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestingIncorrect.tStart = t;  // (not accounting for frame time here)
      textTestingIncorrect.frameNStart = frameN;  // exact frame index
      
      textTestingIncorrect.setAutoDraw(true);
    }

    frameRemains = 0.0 + 0.2 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (textTestingIncorrect.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      textTestingIncorrect.setAutoDraw(false);
    }
    
    // *imageTestingIncorrect* updates
    if (t >= 0.0 && imageTestingIncorrect.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      imageTestingIncorrect.tStart = t;  // (not accounting for frame time here)
      imageTestingIncorrect.frameNStart = frameN;  // exact frame index
      
      imageTestingIncorrect.setAutoDraw(true);
    }

    frameRemains = 0.0 + 0.2 - psychoJS.window.monitorFramePeriod * 0.75;  // most of one frame period left
    if (imageTestingIncorrect.status === PsychoJS.Status.STARTED && t >= frameRemains) {
      imageTestingIncorrect.setAutoDraw(false);
    }
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestingIncorrectComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine && routineTimer.getTime() > 0) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestingIncorrectRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestingIncorrect'-------
    for (const thisComponent of TestingIncorrectComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    return Scheduler.Event.NEXT;
  };
}


var TestEndCodeComponents;
function TestEndCodeRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestEndCode'-------
    t = 0;
    TestEndCodeClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    TestEndCodeComponents = [];
    
    for (const thisComponent of TestEndCodeComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestEndCodeRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestEndCode'-------
    // get current time
    t = TestEndCodeClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestEndCodeComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestEndCodeRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestEndCode'-------
    for (const thisComponent of TestEndCodeComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "TestEndCode" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var BlockSelectComponents;
function BlockSelectRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'BlockSelect'-------
    t = 0;
    BlockSelectClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    // keep track of which components have finished
    BlockSelectComponents = [];
    
    for (const thisComponent of BlockSelectComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function BlockSelectRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'BlockSelect'-------
    // get current time
    t = BlockSelectClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of BlockSelectComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function BlockSelectRoutineEnd() {
  return async function () {
    //------Ending Routine 'BlockSelect'-------
    for (const thisComponent of BlockSelectComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    // the Routine "BlockSelect" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyTestBlockEnd_allKeys;
var TestingBlockEndComponents;
function TestingBlockEndRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'TestingBlockEnd'-------
    t = 0;
    TestingBlockEndClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyTestBlockEnd.keys = undefined;
    keyTestBlockEnd.rt = undefined;
    _keyTestBlockEnd_allKeys = [];
    // keep track of which components have finished
    TestingBlockEndComponents = [];
    TestingBlockEndComponents.push(textTestBlockEnd);
    TestingBlockEndComponents.push(keyTestBlockEnd);
    
    for (const thisComponent of TestingBlockEndComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function TestingBlockEndRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'TestingBlockEnd'-------
    // get current time
    t = TestingBlockEndClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textTestBlockEnd* updates
    if (t >= 0.0 && textTestBlockEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textTestBlockEnd.tStart = t;  // (not accounting for frame time here)
      textTestBlockEnd.frameNStart = frameN;  // exact frame index
      
      textTestBlockEnd.setAutoDraw(true);
    }

    
    // *keyTestBlockEnd* updates
    if (t >= 0.0 && keyTestBlockEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyTestBlockEnd.tStart = t;  // (not accounting for frame time here)
      keyTestBlockEnd.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyTestBlockEnd.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyTestBlockEnd.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyTestBlockEnd.clearEvents(); });
    }

    if (keyTestBlockEnd.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyTestBlockEnd.getKeys({keyList: ['space'], waitRelease: false});
      _keyTestBlockEnd_allKeys = _keyTestBlockEnd_allKeys.concat(theseKeys);
      if (_keyTestBlockEnd_allKeys.length > 0) {
        keyTestBlockEnd.keys = _keyTestBlockEnd_allKeys[_keyTestBlockEnd_allKeys.length - 1].name;  // just the last key pressed
        keyTestBlockEnd.rt = _keyTestBlockEnd_allKeys[_keyTestBlockEnd_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of TestingBlockEndComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function TestingBlockEndRoutineEnd() {
  return async function () {
    //------Ending Routine 'TestingBlockEnd'-------
    for (const thisComponent of TestingBlockEndComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyTestBlockEnd.stop();
    // the Routine "TestingBlockEnd" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


var _keyExperimentEnd_allKeys;
var ExperimentEndComponents;
function ExperimentEndRoutineBegin(snapshot) {
  return async function () {
    TrialHandler.fromSnapshot(snapshot); // ensure that .thisN vals are up to date
    
    //------Prepare to start Routine 'ExperimentEnd'-------
    t = 0;
    ExperimentEndClock.reset(); // clock
    frameN = -1;
    continueRoutine = true; // until we're told otherwise
    // update component parameters for each repeat
    keyExperimentEnd.keys = undefined;
    keyExperimentEnd.rt = undefined;
    _keyExperimentEnd_allKeys = [];
    // keep track of which components have finished
    ExperimentEndComponents = [];
    ExperimentEndComponents.push(textExperimentEnd);
    ExperimentEndComponents.push(keyExperimentEnd);
    
    for (const thisComponent of ExperimentEndComponents)
      if ('status' in thisComponent)
        thisComponent.status = PsychoJS.Status.NOT_STARTED;
    return Scheduler.Event.NEXT;
  }
}


function ExperimentEndRoutineEachFrame() {
  return async function () {
    //------Loop for each frame of Routine 'ExperimentEnd'-------
    // get current time
    t = ExperimentEndClock.getTime();
    frameN = frameN + 1;// number of completed frames (so 0 is the first frame)
    // update/draw components on each frame
    
    // *textExperimentEnd* updates
    if (t >= 0.0 && textExperimentEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      textExperimentEnd.tStart = t;  // (not accounting for frame time here)
      textExperimentEnd.frameNStart = frameN;  // exact frame index
      
      textExperimentEnd.setAutoDraw(true);
    }

    
    // *keyExperimentEnd* updates
    if (t >= 0.0 && keyExperimentEnd.status === PsychoJS.Status.NOT_STARTED) {
      // keep track of start time/frame for later
      keyExperimentEnd.tStart = t;  // (not accounting for frame time here)
      keyExperimentEnd.frameNStart = frameN;  // exact frame index
      
      // keyboard checking is just starting
      psychoJS.window.callOnFlip(function() { keyExperimentEnd.clock.reset(); });  // t=0 on next screen flip
      psychoJS.window.callOnFlip(function() { keyExperimentEnd.start(); }); // start on screen flip
      psychoJS.window.callOnFlip(function() { keyExperimentEnd.clearEvents(); });
    }

    if (keyExperimentEnd.status === PsychoJS.Status.STARTED) {
      let theseKeys = keyExperimentEnd.getKeys({keyList: ['space'], waitRelease: false});
      _keyExperimentEnd_allKeys = _keyExperimentEnd_allKeys.concat(theseKeys);
      if (_keyExperimentEnd_allKeys.length > 0) {
        keyExperimentEnd.keys = _keyExperimentEnd_allKeys[_keyExperimentEnd_allKeys.length - 1].name;  // just the last key pressed
        keyExperimentEnd.rt = _keyExperimentEnd_allKeys[_keyExperimentEnd_allKeys.length - 1].rt;
        // a response ends the routine
        continueRoutine = false;
      }
    }
    
    // check for quit (typically the Esc key)
    if (psychoJS.experiment.experimentEnded || psychoJS.eventManager.getKeys({keyList:['escape']}).length > 0) {
      return quitPsychoJS('The [Escape] key was pressed. Goodbye!', false);
    }
    
    // check if the Routine should terminate
    if (!continueRoutine) {  // a component has requested a forced-end of Routine
      return Scheduler.Event.NEXT;
    }
    
    continueRoutine = false;  // reverts to True if at least one component still running
    for (const thisComponent of ExperimentEndComponents)
      if ('status' in thisComponent && thisComponent.status !== PsychoJS.Status.FINISHED) {
        continueRoutine = true;
        break;
      }
    
    // refresh the screen if continuing
    if (continueRoutine) {
      return Scheduler.Event.FLIP_REPEAT;
    } else {
      return Scheduler.Event.NEXT;
    }
  };
}


function ExperimentEndRoutineEnd() {
  return async function () {
    //------Ending Routine 'ExperimentEnd'-------
    for (const thisComponent of ExperimentEndComponents) {
      if (typeof thisComponent.setAutoDraw === 'function') {
        thisComponent.setAutoDraw(false);
      }
    }
    keyExperimentEnd.stop();
    // the Routine "ExperimentEnd" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset();
    
    return Scheduler.Event.NEXT;
  };
}


function endLoopIteration(scheduler, snapshot) {
  // ------Prepare for next entry------
  return async function () {
    if (typeof snapshot !== 'undefined') {
      // ------Check if user ended loop early------
      if (snapshot.finished) {
        // Check for and save orphaned data
        if (psychoJS.experiment.isEntryEmpty()) {
          psychoJS.experiment.nextEntry(snapshot);
        }
        scheduler.stop();
      } else {
        const thisTrial = snapshot.getCurrentTrial();
        if (typeof thisTrial === 'undefined' || !('isTrials' in thisTrial) || thisTrial.isTrials) {
          psychoJS.experiment.nextEntry(snapshot);
        }
      }
    return Scheduler.Event.NEXT;
    }
  };
}


function importConditions(currentLoop) {
  return async function () {
    psychoJS.importAttributes(currentLoop.getCurrentTrial());
    return Scheduler.Event.NEXT;
    };
}


async function quitPsychoJS(message, isCompleted) {
  // Check for and save orphaned data
  if (psychoJS.experiment.isEntryEmpty()) {
    psychoJS.experiment.nextEntry();
  }
  
  
  
  /* Syntax Error: Fix Python code */
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  psychoJS.window.close();
  psychoJS.quit({message: message, isCompleted: isCompleted});
  
  return Scheduler.Event.QUIT;
}
